@@ -40,6 +40,7 @@ enum cmd_and_opt_values
     oVerbose	= 'v',
     oRuntime    = 'r',
     oComponent  = 'c',
+    oNull       = '0',
     oNoVerbose	= 500,
     oHomedir,
 
@@ -93,6 +94,7 @@ static ARGPARSE_OPTS opts[] =
     { oRuntime, "runtime",  0, N_("activate changes at runtime, if possible") },
     /* hidden options */
     { oHomedir, "homedir", 2, "@" },
+    { oNull, "null", 0, "@" },
     { oNoVerbose, "no-verbose",  0, "@"},
     {0}
   };
@@ -154,7 +156,6 @@ list_dirs (estream_t fp, char **names)
     const char *name;
     const char *(*fnc)(void);
     const char *extra;
-    int special;
   } list[] = {
     { "sysconfdir",         gnupg_sysconfdir, NULL },
     { "bindir",             gnupg_bindir,     NULL },
@@ -162,9 +163,7 @@ list_dirs (estream_t fp, char **names)
     { "libdir",             gnupg_libdir,     NULL },
     { "datadir",            gnupg_datadir,    NULL },
     { "localedir",          gnupg_localedir,  NULL },
-    { "dirmngr-socket",     dirmngr_user_socket_name, NULL, 1 },
-    { "dirmngr-socket",     dirmngr_sys_socket_name,  NULL, 2 },
-    { "dirmngr-sys-socket", dirmngr_sys_socket_name,  NULL, 1 },
+    { "dirmngr-socket",     dirmngr_socket_name, NULL,},
     { "agent-ssh-socket",   gnupg_socketdir,  GPG_AGENT_SSH_SOCK_NAME },
     { "agent-socket",       gnupg_socketdir,  GPG_AGENT_SOCK_NAME },
     { "homedir",            gnupg_homedir,    NULL }
@@ -176,13 +175,6 @@ list_dirs (estream_t fp, char **names)
 
   for (idx = 0; idx < DIM (list); idx++)
     {
-      if (list[idx].special == 1 && dirmngr_user_socket_name ())
-        ;
-      else if (list[idx].special == 2 && !dirmngr_user_socket_name ())
-        ;
-      else if (list[idx].special == 1 || list[idx].special == 2)
-        continue;
-
       s = list[idx].fnc ();
       if (list[idx].extra)
         {
@@ -197,7 +189,10 @@ list_dirs (estream_t fp, char **names)
         {
           for (j=0; names[j]; j++)
             if (!strcmp (names[j], list[idx].name))
-              es_fprintf (fp, "%s\n", s);
+              {
+                es_fputs (s, fp);
+                es_putc (opt.null? '\0':'\n', fp);
+              }
         }
 
       xfree (tmp);
@@ -218,7 +213,7 @@ main (int argc, char **argv)
   early_system_init ();
   gnupg_reopen_std (GPGCONF_NAME);
   set_strusage (my_strusage);
-  log_set_prefix (GPGCONF_NAME, 1);
+  log_set_prefix (GPGCONF_NAME, GPGRT_LOG_WITH_PREFIX);
 
   /* Make sure that our subsystems are ready.  */
   i18n_init();
@@ -241,6 +236,7 @@ main (int argc, char **argv)
         case oVerbose:   opt.verbose++; break;
         case oNoVerbose: opt.verbose = 0; break;
         case oHomedir:   gnupg_set_homedir (pargs.r.ret_str); break;
+        case oNull:      opt.null = 1; break;
 
 	case aListDirs:
         case aListComponents:
